(* -----------------------------------------------------------------------------
  Curves               : 1.0
  Copyright  2017     : Copyright Adirondack Software & Graphics
  Last Modification    : 04-15-2017
  Source File          : uCurves.pas
  ImageEn              : 7.0
  Compiler             : Delphi10Seattle
  PlatForm             : VCL Windows Win32/Win64
  Operating System     : Windows 10
  Original Source Developed By : Roy M Klever
  This file is copyright  W W Miller, 1986-2017
  It may be used without restriction. This code distributed on an "AS IS"
  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
  --------------------------------------------------------------------------- *)
unit uCurves;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Types, Dialogs, StdCtrls, ExtCtrls, ComCtrls, Jpeg, Math, Contnrs,
  imageenview, hyieutils, hyiedefs, ieview, iexBitmaps, iesettings, iexLayers, iexRulers;

type
  PRGB24 = ^TRGB24;

  TRGB24 = record
    B, G, R: Byte;
  end;

  PRGBArray = ^TRGBArray;
  TRGBArray = array [Word] of TRGB24;

  TFormCurves = class(TForm)
    pnlTool: TPanel;
    ImageCurve1: TImage;
    OpenDialog1: TOpenDialog;
    Reset1: TButton;
    Bevel1: TBevel;
    cbPreset: TComboBox;
    labPreset: TLabel;
    btnAdd: TButton;
    cbChannel: TComboBox;
    labChannel: TLabel;
    ImageEnView1: TImageEnView;
    pnlButtons: TPanel;
    Cancel1: TButton;
    Apply1: TButton;
    rbRed: TRadioButton;
    rbGreen: TRadioButton;
    rbBlue: TRadioButton;
    rbRGB: TRadioButton;
    ResetAll1: TButton;
    Open1: TButton;
    Ok1: TButton;
    Undo1: TButton;
    procedure FormCreate(Sender: TObject);
    procedure ImageCurve1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ImageCurve1MouseMove(Sender: TObject; Shift: TShiftState;
      X, Y: Integer);
    procedure ImageCurve1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormDestroy(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure Reset1Click(Sender: TObject);
    procedure ImageEnView1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ImageEnView1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure cbPresetChange(Sender: TObject);
    procedure cbChannelChange(Sender: TObject);
    procedure btnAddClick(Sender: TObject);
    procedure chkAltClick(Sender: TObject);
    procedure Apply1Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure rbRGBClick(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure Cancel1Click(Sender: TObject);
    procedure Ok1Click(Sender: TObject);
    procedure Undo1Click(Sender: TObject);
  private
    { Private declarations }
    procedure SetPandU;
    function GetCurvePoint(i: Integer; v: Single): Single;
    procedure AddPoint(pt: TPoint);
    procedure DelPoint(idx: Integer);
    procedure ApplyCurve(Src: TIEBitmap);
    procedure GetCurvesPreset;
    procedure LoadPreset(idx: Integer);
    procedure DrawHairlines;
  public
    { Public declarations }
    ANumberOfPts: Array [0 .. 3] of Integer;
    APointX, APointY: Array [0 .. 3, 1 .. 32] of Integer;
    AImageIEBitmap: TIEBitmap;
    AImageLoaded: Boolean;
    ACurvesPath: String;
    procedure GetHist;
    procedure SetCurve(s: string);
    procedure MakePreviewImage;
    procedure PaintHistogram(Channel: Byte);
    procedure DrawCurve;
  end;

var
  FormCurves: TFormCurves;

implementation

{$R *.dfm}

Uses Unit1;

var
  APointP, APointU: Array [0 .. 3, 1 .. 32] of Single;
  AHist: Array [0 .. 3, Byte] of Integer;
  AMaxHist: Array [0 .. 3] of Integer;
  ALut: Array [0 .. 3, 0 .. 255] of Byte;
  APoint, AIdx, AGX: Integer;
  AIEBitmap: TIEBitmap;
  AHistBitmap, ACurveBitmap: TBitmap;

function CompareNatural(s1, s2: string): Integer;
  function ExtractNr(n: Integer; var Txt: string): Int64;
  begin
    while (n <= Length(Txt)) and (Txt[n] >= '0') and (Txt[n] <= '9') do
      n := n + 1;
    Result := StrToInt64Def(Copy(Txt, 1, n - 1), 0);
    Delete(Txt, 1, (n - 1));
  end;

var
  iBoolean: Boolean;
begin
  Result := 0;
  s1 := LowerCase(s1);
  s2 := LowerCase(s2);
  if (s1 <> s2) and (s1 <> '') and (s2 <> '') then
  begin
    iBoolean := False;
    while (not iBoolean) do
    begin
      if ((s1[1] >= '0') and (s1[1] <= '9')) and
        ((s2[1] >= '0') and (s2[1] <= '9')) then
        Result := Sign(ExtractNr(1, s1) - ExtractNr(1, s2))
      else
        Result := Sign(Integer(s1[1]) - Integer(s2[1]));
      iBoolean := (Result <> 0) or (Min(Length(s1), Length(s2)) < 2);
      if not iBoolean then
      begin
        Delete(s1, 1, 1);
        Delete(s2, 1, 1);
      end;
    end;
  end;
  if Result = 0 then
  begin
    if (Length(s1) = 1) and (Length(s2) = 1) then
      Result := Sign(Integer(s1[1]) - Integer(s2[1]))
    else
      Result := Sign(Length(s1) - Length(s2));
  end;
end;

function SortMe(List: TStringList; i1, i2: Integer): Integer;
begin
  Result := CompareNatural(List[i1], List[i2]);
end;

function CalcImgSize(w, h, tw, th: Integer): TPoint;
begin
  Result := Point(0, 0);
  if (w = 0) or (h = 0) then
    Exit;
  if (w < tw) and (h < th) then
    Result := Point(w, h)
  else
  begin
    if w > h then
    begin
      if w < tw then
        tw := w;
      Result := Point(tw, Trunc(tw * h / w));
      if Result.Y > th then
        Result := Point(Trunc(th * w / h), th);
    end
    else
    begin
      if h < th then
        th := h;
      Result := Point(Trunc(th * w / h), th);
      if Result.X > tw then
        Result := Point(tw, Trunc(tw * h / w));
    end;
  end;
end;

function Blend(Color1, Color2: TColor; A: Byte): TColor;
var
  ic1, ic2: LongInt;
  iR, iG, iB, iv1, iv2: Byte;
begin
  A := Round(2.56 * A);
  ic1 := ColorToRGB(Color1);
  ic2 := ColorToRGB(Color2);
  iv1 := Byte(ic1);
  iv2 := Byte(ic2);
  iR := A * (iv1 - iv2) shr 8 + iv2;
  iv1 := Byte(ic1 shr 8);
  iv2 := Byte(ic2 shr 8);
  iG := A * (iv1 - iv2) shr 8 + iv2;
  iv1 := Byte(ic1 shr 16);
  iv2 := Byte(ic2 shr 16);
  iB := A * (iv1 - iv2) shr 8 + iv2;
  Result := (iB shl 16) + (iG shl 8) + iR;
end;

function PtInCircle(cx, cy, X, Y, radius: Integer): Boolean;
begin
  Result := ((cx - X) * (cx - X)) + ((cy - Y) * (cy - Y)) <= radius * radius;
end;

procedure WuLine(Src: TBitmap; x1, y1, x2, y2: Integer; Color: TColor);
var
  ic: Cardinal;
  iR, iG, iB: Byte;
  i, dx, dy, X, Y, w, h, a1, a2: Integer;
  dxi, dyi, gradient: Integer;
  iLine: array of PRGBArray;

  function BlendPixel(X, Y, A: Integer): TRGB24;
  begin
    Result.R := A * (iR - iLine[Y][X].R) shr 8 + iLine[Y][X].R;
    Result.G := A * (iG - iLine[Y][X].G) shr 8 + iLine[Y][X].G;
    Result.B := A * (iB - iLine[Y][X].B) shr 8 + iLine[Y][X].B;
  end;

begin
  ic := ColorToRGB(Color);
  iR := ic and 255;
  iG := (ic shr 8) and 255;
  iB := (ic shr 16) and 255;
  w := Src.Width;
  h := Src.Height;
  if (x1 = x2) or (y1 = y2) then
  begin
    Src.Canvas.Pen.Color := Color;
    Src.Canvas.MoveTo(x1, y1);
    Src.Canvas.LineTo(x2, y2);
    Exit;
  end;
  // make an array of source scanlines to speed up the rendering
  SetLength(iLine, Src.Height);
  iLine[0] := (Src.Scanline[0]);
  i := Integer(Src.Scanline[1]) - Integer(iLine[0]);
  for Y := 1 to Src.Height - 1 do
    iLine[Y] := PRGBArray(Integer(iLine[Y - 1]) + i);
  dx := abs(x2 - x1);
  dy := abs(y2 - y1);
  if dx > dy then
  begin // horizontal or vertical
    if y2 > y1 then
      dy := -dy;
    gradient := dy shl 8 div dx;
    if x2 < x1 then
    begin
      i := x1;
      x1 := x2;
      x2 := i;
      dyi := y2 shl 8;
    end
    else
    begin
      dyi := y1 shl 8;
      gradient := -gradient;
    end;
    if x1 >= w then
      x2 := w - 1;
    for X := x1 to x2 do
    begin
      Y := dyi shr 8;
      if (X < 0) or (Y < 0) or (Y > h - 2) then
        Inc(dyi, gradient)
      else
      begin
        a1 := dyi - Y shl 8;
        a2 := 256 - a1;
        iLine[Y][X] := BlendPixel(X, Y, a2);
        iLine[Y + 1][X] := BlendPixel(X, Y + 1, a1);
        Inc(dyi, gradient);
      end;
    end;
  end
  else
  begin
    if x2 > x1 then
      dx := -dx;
    gradient := dx shl 8 div dy;
    if y2 < y1 then
    begin
      i := y1;
      y1 := y2;
      y2 := i;
      dxi := x2 shl 8;
    end
    else
    begin
      dxi := x1 shl 8;
      gradient := -gradient;
    end;
    if y2 >= h then
      y2 := h - 1;
    for Y := y1 to y2 do
    begin
      X := dxi shr 8;
      if (Y < 0) or (X < 0) or (X > w - 2) then
        Inc(dxi, gradient)
      else
      begin
        a1 := dxi - X shl 8;
        a2 := 256 - a1;
        iLine[Y][X] := BlendPixel(X, Y, a2);
        iLine[Y][X + 1] := BlendPixel(X + 1, Y, a1);
        Inc(dxi, gradient);
      end;
    end;
  end;
end;

procedure TFormCurves.SetPandU;
var
  i: Integer;
  d, w: array [1 .. 32] of Single;
begin
  for i := 2 to ANumberOfPts[AIdx] - 1 do
    d[i] := 2 * (APointX[AIdx, i + 1] - APointX[AIdx, i - 1]);
  for i := 1 to ANumberOfPts[AIdx] - 1 do
    APointU[AIdx, i] := APointX[AIdx, i + 1] - APointX[AIdx, i];
  for i := 2 to ANumberOfPts[AIdx] - 1 do
    w[i] := 6 * ((APointY[AIdx, i + 1] - APointY[AIdx, i]) / APointU[AIdx, i] -
      (APointY[AIdx, i] - APointY[AIdx, i - 1]) / APointU[AIdx, i - 1]);
  for i := 2 to ANumberOfPts[AIdx] - 2 do
  begin
    w[i + 1] := w[i + 1] - w[i] * APointU[AIdx, i] / d[i];
    d[i + 1] := d[i + 1] - APointU[AIdx, i] * APointU[AIdx, i] / d[i];
  end;
  APointP[AIdx, 1] := 0;
  for i := ANumberOfPts[AIdx] - 1 downto 2 do
    APointP[AIdx, i] := (w[i] - APointU[AIdx, i] * APointP[AIdx, i + 1]) / d[i];
  APointP[AIdx, ANumberOfPts[AIdx]] := 0;
end;

procedure TFormCurves.Undo1Click(Sender: TObject);
var
  i, j: Integer;
begin
    ImageEnView1.Proc.Undo();
    ImageEnView1.Proc.ClearUndo;
    ImageEnView1.Update;
    Form1.ImageEnView.Proc.Undo();
    Form1.ImageEnView.Proc.ClearAllUndo;
    Form1.ImageEnView.Update;
    APoint := -1;
    for i := 0 to 3 do
    begin
      // Reset values
      ANumberOfPts[i] := 2;
      APointX[i, 1] := 0;
      APointX[i, 2] := 255;
      APointY[i, 1] := 0;
      APointY[i, 2] := 255;
      for j := 0 to 255 do
        ALut[i, j] := j;
    end;
    DrawCurve;
    Undo1.Enabled := Form1.ImageEnView.Proc.CanUndo;
    Form1.ImageEnView.IEBitmap.Modified := False;
    Form1.UpdateFrame;
    Form1.UpdateStatusBar;
    Apply1.Enabled := True;
end;

function TFormCurves.GetCurvePoint(i: Integer; v: Single): Single;
var
  t0, t1: Single;
begin
  t0 := (v - APointX[AIdx, i]) / APointU[AIdx, i];
  t1 := 1 - t0;
  Result := t0 * APointY[AIdx, i + 1] + t1 * APointY[AIdx, i] + APointU[AIdx, i] *
    APointU[AIdx, i] * ((t0 * t0 * t0 - t0) * APointP[AIdx, i + 1] + (t1 * t1 * t1 - t1)
    * APointP[AIdx, i]) / 6;
end;

procedure TFormCurves.AddPoint(pt: TPoint);
var
  i, X: Integer;
begin
  i := 1;
  while (i <= ANumberOfPts[AIdx]) and (pt.X > APointX[AIdx, i]) do
    i := i + 1;
  if i <= ANumberOfPts[AIdx] + 1 then
  begin
    for X := 31 downto i do
    begin
      APointX[AIdx, X + 1] := APointX[AIdx, X];
      APointY[AIdx, X + 1] := APointY[AIdx, X];
    end;
    APointX[AIdx, i] := pt.X;
    APointY[AIdx, i] := pt.Y;
    APoint := i;
    ANumberOfPts[AIdx] := ANumberOfPts[AIdx] + 1;
  end;
end;

procedure TFormCurves.DelPoint(idx: Integer);
var
  X: Integer;
begin
  if ANumberOfPts[AIdx] = 2 then
  begin
    ShowMessage('At least two points must exist.');
    Exit;
  end;
  if (idx > 0) and (idx <= ANumberOfPts[AIdx]) then
  begin
    for X := idx to 31 do
    begin
      APointX[AIdx, X] := APointX[AIdx, X + 1];
      APointY[AIdx, X] := APointY[AIdx, X + 1];
    end;
    APoint := -1;
    ANumberOfPts[AIdx] := ANumberOfPts[AIdx] - 1;
    DrawCurve;
  end;
end;

procedure TFormCurves.ImageCurve1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  i: Integer;
  iPoint: TPoint;
begin
  if (X <= 8) or (X >= 263) then
    Exit;
  X := X - 8;
  Y := Y - 8;
  iPoint.X := X;
  iPoint.Y := 255 - Y;
  if iPoint.X < 0 then
    iPoint.X := 0
  else if iPoint.X > 255 then
    iPoint.X := 255;
  if iPoint.Y < 0 then
    iPoint.Y := 0
  else if iPoint.Y > 255 then
    iPoint.Y := 255;
  APoint := -1;

  i := 1;
  while (i <= ANumberOfPts[AIdx]) and (not PtInCircle(APointX[AIdx, i], APointY[AIdx, i], iPoint.X,
    iPoint.Y, 5)) do
    Inc(i);

  if i <= ANumberOfPts[AIdx] then
    APoint := i;

  if (ssLeft in Shift) then
  begin
    if (APoint <> -1) then
    begin
      if (APoint > 0) and (APoint < ANumberOfPts[AIdx]) then
      begin
        if (iPoint.X > APointX[AIdx, APoint - 1]) and (iPoint.X < APointX[AIdx, APoint + 1]) then
          APointX[AIdx, APoint] := iPoint.X;
        APointY[AIdx, APoint] := iPoint.Y;
      end;
    end
    else if ANumberOfPts[AIdx] < 31 then
      AddPoint(iPoint);
  end
  else if (ssRight in Shift) then
    DelPoint(APoint);
  DrawCurve;
  Apply1.Enabled := True;
end;

procedure TFormCurves.ImageCurve1MouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
var
  i: Integer;
  iPoint: TPoint;
begin
  AGX := X - 8;
  if AGX > 255 then
    AGX := 255;

  if not(ssLeft in Shift) then
    DrawHairlines
  else
  begin
    X := X - 8;
    Y := Y - 8;
    iPoint.X := X;
    iPoint.Y := 255 - Y;
    if iPoint.X < 0 then
      iPoint.X := 0
    else if iPoint.X > 255 then
      iPoint.X := 255;
    if iPoint.Y < 0 then
      iPoint.Y := 0
    else if iPoint.Y > 255 then
      iPoint.Y := 255;
    if APoint = -1 then
    begin
      i := 1;
      while (i <= ANumberOfPts[AIdx]) and
        (not PtInCircle(APointX[AIdx, i], APointY[AIdx, i], iPoint.X, iPoint.Y, 5)) do
        Inc(i);
      if i <= ANumberOfPts[AIdx] then
        APoint := i;
    end;
    if (ssLeft in Shift) and (APoint <> -1) then
    begin
      if (APoint > 1) and (APoint < ANumberOfPts[AIdx]) then
      begin
        if (iPoint.X > APointX[AIdx, APoint - 1]) and (iPoint.X < APointX[AIdx, APoint + 1]) then
          APointX[AIdx, APoint] := iPoint.X;
      end
      else
      begin
        if (APoint = 1) and (iPoint.X < APointX[AIdx, APoint + 1]) then
          APointX[AIdx, APoint] := iPoint.X;
        if (APoint = ANumberOfPts[AIdx]) and (iPoint.X > APointX[AIdx, APoint - 1]) then
          APointX[AIdx, APoint] := iPoint.X;
      end;
      APointY[AIdx, APoint] := iPoint.Y;
    end;
    DrawCurve;
  end;
end;

procedure TFormCurves.ImageCurve1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  APoint := -1;
  DrawCurve;
end;

procedure TFormCurves.ImageEnView1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  ImageEnView1.IEBitmap.Assign(AIEBitmap);
  ImageEnView1.Refresh;
end;

procedure TFormCurves.ImageEnView1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  ImageEnView1.IEBitmap.Assign(AIEBitmap);
  ImageEnView1.IEBitmap.Assign(AIEBitmap);
  ImageEnView1.IEBitmap.Assign(AIEBitmap);
  ImageEnView1.LockPaint;
  ApplyCurve(ImageEnView1.IEBitmap);
  ImageEnView1.UnLockPaint;
  ImageEnView1.Paint;
end;

procedure TFormCurves.ApplyCurve(Src: TIEBitmap);
var
  iRow: PRGBArray;
  iFill, X, Y, w, h: Integer;
  iLut: Array [0 .. 2, 0 .. 255] of Byte;
begin
  if not AImageLoaded then
    Exit;
  for X := 0 to 255 do
  begin
    iLut[0, X] := ALut[0, ALut[1, X]];
    iLut[1, X] := ALut[0, ALut[2, X]];
    iLut[2, X] := ALut[0, ALut[3, X]];
  end;
  iRow := PRGBArray(Src.Scanline[0]);
  iFill := Integer(Src.Scanline[1]) - Integer(iRow);
  w := Src.Width - 1;
  h := Src.Height - 1;
  for Y := 0 to h do
  begin
    for X := 0 to w do
    begin
      iRow[X].R := iLut[0, iRow[X].R];
      iRow[X].G := iLut[1, iRow[X].G];
      iRow[X].B := iLut[2, iRow[X].B];
    end;
    iRow := PRGBArray(Integer(iRow) + iFill);
  end;
end;

procedure TFormCurves.GetHist;
var
  iRow: PRGBArray;
  X, Y, RFill: Integer;
  iIEBitmap: TIEBitmap;
  iRGB: TRGB24;
  iR, iG, iB, l: Byte;
begin
  if not AImageLoaded then
    Exit;
  for Y := 0 to 3 do
  begin
    AMaxHist[Y] := 0;
    for X := 0 to 255 do
      AHist[Y, X] := 0;
  end;
  iIEBitmap := ImageEnView1.IEBitmap;
  iRow := PRGBArray(iIEBitmap.Scanline[0]);
  RFill := Integer(iIEBitmap.Scanline[1]) - Integer(iRow);
  for Y := 0 to iIEBitmap.Height - 1 do
  begin
    for X := 0 to iIEBitmap.Width - 1 do
    begin
      iRGB := iRow[X];
      iR := iRGB.R;
      iG := iRGB.G;
      iB := iRGB.B;
      l := (iR + iG + iB) div 3;
      AHist[0, l] := AHist[0, l] + 1;
      AHist[1, iR] := AHist[1, iR] + 1;
      AHist[2, iG] := AHist[2, iG] + 1;
      AHist[3, iB] := AHist[3, iB] + 1;
    end;
    iRow := PRGBArray(Integer(iRow) + RFill);
  end;
  for Y := 0 to 3 do
    for X := 0 to 255 do
      if AHist[Y, X] > AMaxHist[Y] then
        AMaxHist[Y] := AHist[Y, X];
end;

procedure TFormCurves.PaintHistogram(Channel: Byte);
var
  i, j, k: Integer;
  iCanvas: TCanvas;
  iColor, iColor1: TColor;
begin
  iCanvas := AHistBitmap.Canvas;

  iCanvas.Brush.Color := clWhite;
  iCanvas.FillRect(AHistBitmap.Canvas.ClipRect);

  // Paint histogram
  case Channel of
    0:
      iColor := RGB(192, 192, 192);
    1:
      iColor := RGB(255, 190, 190);
    2:
      iColor := RGB(190, 220, 190);
    3:
      iColor := RGB(190, 190, 255);
  else
    iColor := RGB(190, 190, 255);
  end;

  iColor1:= Blend(iColor, clWhite, 30);
  iCanvas.Pen.Color := iColor;
  j := MulDiv(AHist[Channel, 0], 230, AMaxHist[Channel]);
  iCanvas.MoveTo(8, 255 + 8);
  iCanvas.LineTo(8, (255 + 8) - j);
  k := (255 + 8) - j;
  for i := 1 to 255 do
  begin
    j := (255 + 8) - MulDiv(AHist[Channel, i], 230, AMaxHist[Channel]);
    iCanvas.Pen.Color := iColor1;
    iCanvas.MoveTo(i + 8, 255 + 8);
    iCanvas.LineTo(i + 8, j);
    WuLine(AHistBitmap, 8 + (i - 1), k, 8 + i, j, iColor);
    k := j;
  end;
  // Histogram done...

  // Paint guidelines
  iCanvas.Pen.Color := clGray;
  iCanvas.Brush.Style := bsClear;
  iCanvas.Rectangle(Rect(8, 8, 264, 264));
  iCanvas.Pen.Color := clSilver;
  iCanvas.Pen.Style := psDot;
  for i := 1 to 3 do
  begin
    j := 8 + (i * 64);
    iCanvas.MoveTo(8, j);
    iCanvas.LineTo(264, j);
    iCanvas.MoveTo(j, 8);
    iCanvas.LineTo(j, 264);
  end;
  iCanvas.Pen.Style := psSolid;
  iCanvas.Pen.Color := clSilver;
  iCanvas.MoveTo(8, 263);
  iCanvas.LineTo(263, 8);
  // Guidelines done...
end;

procedure TFormCurves.rbRGBClick(Sender: TObject);
begin
  AIdx := (Sender as TRadioButton).Tag;
  PaintHistogram(AIdx);
  DrawCurve;
end;

procedure TFormCurves.DrawHairlines;
var
  iCanvas: TCanvas;
begin
  ImageCurve1.Picture.Bitmap.Assign(ACurveBitmap);
  if (AGX >= 0) and (AGX <= 255) then
  begin
    iCanvas := ImageCurve1.Picture.Bitmap.Canvas;
    iCanvas.Pen.Color := clSilver;
    iCanvas.MoveTo(8, 263 - ALut[AIdx, AGX]);
    iCanvas.LineTo(263, 263 - ALut[AIdx, AGX]);
    iCanvas.MoveTo(8 + AGX, 8);
    iCanvas.LineTo(8 + AGX, 263);
  end;
  ImageCurve1.Refresh;
end;

procedure TFormCurves.DrawCurve;
var
  i, j, X, Y, x1, x2, xpos, ypos: Integer;
  iCanvas: TCanvas;
begin
  SetPandU;

  ACurveBitmap.Assign(AHistBitmap);
  iCanvas := ACurveBitmap.Canvas;

  // Paint points and curve
  iCanvas.Pen.Color := clBlack;
  iCanvas.Brush.Style := bsClear;
  for i := 1 to ANumberOfPts[AIdx] do
    iCanvas.Rectangle(8 + APointX[AIdx, i] - 3, 263 - APointY[AIdx, i] - 3,
      8 + APointX[AIdx, i] + 4, 263 - APointY[AIdx, i] + 4);

  xpos := 0;
  ypos := 0;
  X := 8 + APointX[AIdx, 1];
  Y := 263 - APointY[AIdx, 1];
  iCanvas.MoveTo(8, Y);
  if APointX[AIdx, 1] > 0 then
    iCanvas.LineTo(X, Y);
  for i := 1 to ANumberOfPts[AIdx] - 1 do
  begin
    x1 := APointX[AIdx, i];
    x2 := APointX[AIdx, i + 1];
    for j := x1 to x2 do
    begin
      xpos := j;
      ypos := Trunc(GetCurvePoint(i, xpos));
      if ypos < 0 then
        ypos := 0
      else if ypos > 255 then
        ypos := 255;
      WuLine(ACurveBitmap, X, Y, 8 + xpos, 263 - ypos, clBlack);
      ALut[AIdx, xpos] := ypos;
      X := 8 + xpos;
      Y := 263 - ypos;
    end;
  end;
  iCanvas.MoveTo(8 + xpos, 263 - ypos);
  if APointX[AIdx, ANumberOfPts[AIdx]] < 255 then
    iCanvas.LineTo(263, 263 - APointY[AIdx, ANumberOfPts[AIdx]]);
  if APointX[AIdx, 1] > 0 then
    for i := 0 to APointX[AIdx, 1] - 1 do
      ALut[AIdx, i] := APointY[AIdx, 1];
  if APointX[AIdx, ANumberOfPts[AIdx]] < 255 then
    for i := APointX[AIdx, ANumberOfPts[AIdx]] + 1 to 255 do
      ALut[AIdx, i] := ypos;
  // Curve and points done...

  DrawHairlines;

  ImageEnView1.IEBitmap.Assign(AIEBitmap);
  ImageEnView1.LockPaint;
  ApplyCurve(ImageEnView1.IEBitmap);
  ImageEnView1.UnLockPaint;
  ImageEnView1.Paint;
end;

procedure TFormCurves.MakePreviewImage;
var
  iPoint: TPoint;
begin
  iPoint := CalcImgSize(AImageIEBitmap.Width, AImageIEBitmap.Height, ImageEnView1.Width,
    ImageEnView1.Height);
  if (iPoint.X < 2) or (iPoint.Y < 2) then
    Exit;
  AIEBitmap.Assign(AImageIEBitmap);
  AIEBitmap.Resample(iPoint.X, iPoint.Y, rfWICFant, True);
  ImageEnView1.IEBitmap.Assign(AIEBitmap);
  DrawCurve;
end;

procedure TFormCurves.btnAddClick(Sender: TObject);
var
  i, j, k: Integer;
  iText: String;
  iAcv: Array [0 .. 255] of Byte;
  iStream: TFileStream;
  ACVFile: String;
begin
  iText := InputBox('Give the curve a name', 'Please type a name ', 'User');
  for i := 0 to 255 do
    iAcv[i] := 0;
  if iText <> '' then
  begin
    j := 1;
    iAcv[j] := 4;
    Inc(j, 2);
    iAcv[j] := 5;
    Inc(j, 2);
    for i := 0 to 3 do
    begin
      iAcv[j] := ANumberOfPts[i];
      Inc(j, 2);
      for k := 1 to ANumberOfPts[i] do
      begin
        iAcv[j] := APointY[i, k];
        Inc(j, 2);
        iAcv[j] := APointX[i, k];
        Inc(j, 2);
      end;
    end;
    iAcv[j] := 2;
    Inc(j, 2);
    iAcv[j] := 0;
    Inc(j, 2);
    iAcv[j] := 0;
    Inc(j, 2);
    iAcv[j] := 255;
    Inc(j, 2);
    iAcv[j] := 255;
    Inc(j, 2);
    ACVFile := ACurvesPath + iText + '.acv';
    iStream := TFileStream.Create(ACVFile, fmCreate or fmOpenWrite or
      fmShareDenyWrite);
    try
      iStream.WriteBuffer(iAcv, j - 1);
    finally
      iStream.Free;
    end;
    if cbPreset.Items.IndexOf(iText) <> -1 then
      cbPreset.Items.Add(iText);
    cbPreset.ItemIndex := cbPreset.Items.IndexOf(iText);
  end;
end;

procedure TFormCurves.Apply1Click(Sender: TObject);
begin
   Form1.ImageEnView.Proc.SaveUndo
      ('Curve ' + IntToStr(Form1.ImageEnView.Proc.UndoCount));
    Form1.Undo1.Hint := Form1.ImageEnView.Proc.UndoCaptions[0];
    Undo1.Enabled := Form1.ImageEnView.Proc.CanUndo;
    ApplyCurve(Form1.ImageEnView.IEBitmap);
    Form1.ImageEnView.Update;
    Form1.ImageEnView.IEBitmap.Modified := True;
    Apply1.Enabled := False;
    Form1.UpdateFrame;
    Form1.UpdateGUI;
    Form1.UpdateStatusBar;
end;

procedure TFormCurves.Cancel1Click(Sender: TObject);
begin
  Close;
end;

procedure TFormCurves.Ok1Click(Sender: TObject);
begin
  if (not Form1.ImageEnView.IEBitmap.Modified) then
  begin
    Form1.ImageEnView.Proc.SaveUndo
      ('Curve ' + IntToStr(Form1.ImageEnView.Proc.UndoCount));
    Form1.Undo1.Hint := Form1.ImageEnView.Proc.UndoCaptions[0];
    ApplyCurve(Form1.ImageEnView.IEBitmap);
    Form1.ImageEnView.Update;
    Form1.ImageEnView.IEBitmap.Modified := True;
    Form1.UpdateGUI;
    Form1.UpdateFrame;
    Form1.UpdateStatusBar;
  end;
end;

procedure TFormCurves.Open1Click(Sender: TObject);
begin
  if OpenDialog1.Execute then
  begin
    ImageEnView1.IO.LoadFromFile(OpenDialog1.Filename);
    AImageIEBitmap.Assign(ImageEnView1.IEBitmap);
    AImageIEBitmap.PixelFormat := ie24RGB;
    ImageEnView1.IEBitmap.Assign(AImageIEBitmap);
    MakePreviewImage;
    AImageLoaded := True;
    ImageEnView1.Enabled := AImageLoaded;
    GetHist;
    PaintHistogram(AIdx);
    DrawCurve;
  end;
end;

procedure TFormCurves.Reset1Click(Sender: TObject);
var
  i, j: Integer;
begin
  APoint := -1;
  if (Sender as TButton).Tag = 0 then
  begin
    // Reset values
    ANumberOfPts[AIdx] := 2;
    APointX[AIdx, 1] := 0;
    APointX[AIdx, 2] := 255;
    APointY[AIdx, 1] := 0;
    APointY[AIdx, 2] := 255;
    for j := 0 to 255 do
      ALut[AIdx, j] := j;
  end
  else
  begin
    for i := 0 to 3 do
    begin
      // Reset values
      ANumberOfPts[i] := 2;
      APointX[i, 1] := 0;
      APointX[i, 2] := 255;
      APointY[i, 1] := 0;
      APointY[i, 2] := 255;
      for j := 0 to 255 do
        ALut[i, j] := j;
    end;
  end;
  DrawCurve;
end;

procedure TFormCurves.cbChannelChange(Sender: TObject);
begin
  if cbChannel.ItemIndex = 3 then
    AIdx := 0
  else
    AIdx := cbChannel.ItemIndex + 1;
  PaintHistogram(AIdx);
  DrawCurve;
end;

procedure TFormCurves.cbPresetChange(Sender: TObject);
begin
  if cbPreset.ItemIndex <> -1 then
    LoadPreset(cbPreset.ItemIndex);
end;

procedure TFormCurves.chkAltClick(Sender: TObject);
begin
  MakePreviewImage;
end;

procedure TFormCurves.SetCurve(s: string);
var
  ch, cnt, l, m, n, i, X, Y: Integer;
  iStringList, iLStringList: TStringList;
begin
  Y := 255;
  m := AIdx;

  // Clear old values
  for i := 0 to 3 do
  begin
    for n := 1 to 32 do
    begin
      APointP[i, n] := 0;
      APointU[i, n] := 0;
    end;
  end;

  iStringList := TStringList.Create;
  iLStringList := TStringList.Create;
  // Split curve values into 4 strings
  iStringList.Text := Trim(StringReplace(s, '|', ''#13''#10'', [rfReplaceAll]));

  // For each channel we need to get the values in so we can calculate the LUTs
  for ch := 0 to iStringList.Count - 1 do
  begin
    iLStringList.Text := Trim(StringReplace(iStringList[ch], ',', ''#13''#10'', [rfReplaceAll]));
    cnt := iLStringList.Count div 2;
    ANumberOfPts[ch] := cnt;
    for n := 0 to cnt - 1 do
    begin
      l := (n * 2);
      APointX[ch, n + 1] := StrToIntDef(iLStringList[l], 0);
      APointY[ch, n + 1] := StrToIntDef(iLStringList[l + 1], 0);
    end;
    AIdx := ch;
    SetPandU;
    for i := 1 to cnt - 1 do
    begin
      for X := APointX[ch, i] to APointX[ch, i + 1] do
      begin
        Y := Trunc(GetCurvePoint(i, X));
        if Y < 0 then
          Y := 0
        else if Y > 255 then
          Y := 255;
        ALut[ch, X] := Y;
      end;
    end;
    if APointX[ch, 1] > 0 then
      for i := 0 to APointX[AIdx, 1] - 1 do
        ALut[AIdx, i] := APointY[ch, 1];
    if APointX[ch, ANumberOfPts[ch]] < 255 then
      for i := APointX[ch, ANumberOfPts[AIdx]] + 1 to 255 do
        ALut[ch, i] := Y;
  end;

  // Clean up and draw the new curve
  iLStringList.Free;
  iStringList.Free;
  AIdx := m;
  DrawCurve;
end;

procedure TFormCurves.LoadPreset(idx: Integer);
var
  X, Y, i, j, k, m, n, p, q, cnt: Integer;
  iFileArr, iSimple: Array [0 .. 255] of Byte;
  iStream: TFileStream;
  iACVFile: String;
begin
  q := AIdx;
  iACVFile := ACurvesPath + cbPreset.Items[idx] + '.acv';
  iStream := TFileStream.Create(iACVFile, fmOpenRead or fmShareDenyWrite);
  try
    i := iStream.Size;
    iStream.ReadBuffer(iFileArr, i);
  finally
    iStream.Free;
  end;
  i := i div 2;
  n := 1;
  for j := 0 to i - 1 do
  begin
    iSimple[j] := iFileArr[n];
    n := n + 2;
  end;
  if (iSimple[0] <> 4) then
  begin
    ShowMessage('This file version is not supported. Sorry!');
    Exit;
  end;
  // Clear old values
  for i := 0 to 3 do
  begin
    for n := 1 to 32 do
    begin
      APointP[i, n] := 0;
      APointU[i, n] := 0;
    end;
  end;
  k := 2;
  for m := 0 to 3 do
  begin
    cnt := iSimple[k];
    i := iSimple[k] * 2;
    ANumberOfPts[m] := cnt;
    n := k + 1;
    for j := 1 to cnt do
    begin
      APointX[m, j] := iSimple[n + 1];
      APointY[m, j] := iSimple[n];
      n := n + 2;
    end;
    AIdx := m;
    SetPandU;
    for p := 1 to cnt - 1 do
    begin
      for X := APointX[m, p] to APointX[m, p + 1] do
      begin
        Y := Trunc(GetCurvePoint(p, X));
        if Y < 0 then
          Y := 0
        else if Y > 255 then
          Y := 255;
        ALut[m, X] := Y;
      end;
    end;
    if APointX[m, 1] > 0 then
      for p := 0 to APointX[m, 1] - 1 do
        ALut[m, p] := APointY[m, 1];
    if APointX[m, ANumberOfPts[m]] < 255 then
      for p := APointX[m, ANumberOfPts[m]] + 1 to 255 do
        ALut[m, p] := 255;
    k := k + i + 1;
  end;
  AIdx := q;
  DrawCurve;
end;

{$WARN SYMBOL_PLATFORM OFF}

procedure TFormCurves.GetCurvesPreset;
var
  iFiles: TStringList;
  iSearchRec: TSearchRec;
  i: Integer;
begin
  if not DirectoryExists(ACurvesPath) then
    CreateDir(ACurvesPath)
  else
  begin
    cbPreset.Items.Clear;
    iFiles := TStringList.Create;
    try
      if FindFirst(ACurvesPath + '*.*', faArchive, iSearchRec) = 0 then
      begin
        repeat
          iFiles.Add(iSearchRec.Name);
        until FindNext(iSearchRec) <> 0;
        FindClose(iSearchRec);
      end;
      if iFiles.Count > 0 then
      begin
        iFiles.CustomSort(SortMe);
        cbPreset.Items.Clear;
        for i := 0 to iFiles.Count - 1 do
          cbPreset.Items.Add(ChangeFileExt(iFiles[i], ''));
      end;
    finally
      iFiles.Free;
    end;
  end;
end;
{$WARN SYMBOL_PLATFORM ON}

procedure TFormCurves.FormCreate(Sender: TObject);
var
  i, j: Integer;
  iBitmap: TBitmap;
begin
  AHistBitmap := TBitmap.Create;
  AHistBitmap.PixelFormat := pf24bit;
  AHistBitmap.Width := 272;
  AHistBitmap.Height := 272;
  ACurveBitmap := TBitmap.Create;
  ACurveBitmap.Assign(AHistBitmap);
  AImageIEBitmap := TIEBitmap.Create;
  AIEBitmap := TIEBitmap.Create;
  AIEBitmap.PixelFormat := ie24RGB;
  ImageEnView1.IEBitmap.Width := 1;
  ImageEnView1.IEBitmap.Height := 1;
  ImageEnView1.IEBitmap.Canvas.Pixels[0, 0] := $00C4C4C4;
  ImageEnView1.MouseWheelParams.Action := iemwNone;
  iBitmap := TBitmap.Create;
  iBitmap.PixelFormat := pf24bit;
  iBitmap.Width := 272;
  iBitmap.Height := 272;
  ImageCurve1.Picture.Bitmap.Assign(iBitmap);
  iBitmap.Free;
  AIdx := 0;
  APoint := -1;
  // Init start values
  for i := 0 to 3 do
  begin
    ANumberOfPts[i] := 2;
    APointX[i, 1] := 0;
    APointX[i, 2] := 255;
    APointY[i, 1] := 0;
    APointY[i, 2] := 255;
    for j := 0 to 255 do
      ALut[i, j] := j;
  end;
  ACurvesPath := ExtractFileDir(Application.ExeName) + '\Curves\';
  GetCurvesPreset;
  Form1.ImageEnView.IEBitmap.Modified := False;
end;

procedure TFormCurves.FormDestroy(Sender: TObject);
begin
  ACurveBitmap.Free;
  AHistBitmap.Free;
  AImageIEBitmap.Free;
  AIEBitmap.Free;
end;

procedure TFormCurves.FormResize(Sender: TObject);
begin
  if AImageLoaded then
    MakePreviewImage;
end;

procedure TFormCurves.FormShow(Sender: TObject);
begin
  if AImageLoaded then
    MakePreviewImage;
  GetHist;
  PaintHistogram(AIdx);
  DrawCurve;
end;

end.
