(* -----------------------------------------------------------------------------
  New Image            : 1.0
  Copyright  2017     : Copyright Adirondack Software & Graphics
  Last Modification    : 05-15-2017
  Source File          : uNewImage.pas
  ImageEn              : 7.0
  Compiler             : Delphi Seattle
  PlatForm             : VCL Windows Win32/Win64
  Operating System     : Windows 10
  This file is copyright  W W Miller, 1986-2017
  It may be used without restriction. This code distributed on an "AS IS"
  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
  --------------------------------------------------------------------------- *)
unit uNewImage;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, Menus,
  Buttons, ExtCtrls, ComCtrls, Dialogs, WinXCtrls;

type
  TFormNewImage = class(TForm)
    ColorDepth1: TRadioGroup;
    GroupBox1: TGroupBox;
    GroupBox3: TGroupBox;
    GroupBox4: TGroupBox;
    RadioButton1: TRadioButton;
    RadioButton3: TRadioButton;
    RadioButton4: TRadioButton;
    RadioButton6: TRadioButton;
    RadioButton7: TRadioButton;
    RadioButton8: TRadioButton;
    RadioButton9: TRadioButton;
    RadioButton5: TRadioButton;
    RadioButton2: TRadioButton;
    RadioButton10: TRadioButton;
    Width1: TEdit;
    Height1: TEdit;
    UpDown1: TUpDown;
    UpDown2: TUpDown;
    Status1: TLabel;
    Label1: TLabel;
    ColorBox1: TColorBox;
    FileType1: TComboBox;
    OKBtn: TButton;
    CancelBtn: TButton;
    procedure RadioButton4Click(Sender: TObject);
    procedure RadioButton1Click(Sender: TObject);
    procedure RadioButton3Click(Sender: TObject);
    procedure cxRadioButton4Click(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure RadioButton6Click(Sender: TObject);
    procedure RadioButton7Click(Sender: TObject);
    procedure RadioButton8Click(Sender: TObject);
    procedure ColorDepth1Click(Sender: TObject);
    procedure FileType1Change(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FileType1PropertiesChange(Sender: TObject);
    procedure RadioGroup1PropertiesChange(Sender: TObject);
    procedure Width1PropertiesChange(Sender: TObject);
    procedure Height1PropertiesChange(Sender: TObject);
    procedure RadioButton2Click(Sender: TObject);
    procedure RadioButton10Click(Sender: TObject);
    procedure RadioButton5Click(Sender: TObject);
  private
    procedure SetImageDescription;
    procedure SetFileType;
    procedure SetPixelFormat;
    procedure SetColor;
    procedure SetImageDimensions;
    { Private declarations }
  public
    { Public declarations }
    AImageWidth: integer;
    AImageHeight: integer;
    APixelFormat: TPixelFormat;
    ABitDepth: integer;
    AColor: TColor;
    AFileType: integer;
    AFileTypeStr: string;
    AFileExt: string;
    AImageType: integer;
    ABitsPerSample: integer;
    ASamplesPerPixel: integer;
    property BitDepth: integer read ABitDepth write ABitDepth;
    property Color: TColor read AColor write AColor;
    property FileType: integer read AFileType write AFileType;
    property FileTypeStr: string read AFileTypeStr write AFileTypeStr;
    property FileExt: string read AFileExt write AFileExt;
    property ImageHeight: integer read AImageHeight write AImageHeight;
    property ImageWidth: integer read AImageWidth write AImageWidth;
    property PixelFormat: TPixelFormat read APixelFormat write APixelFormat;
    property BitsPerSample: integer read ABitsPerSample write ABitsPerSample;
    property SamplesPerPixel: integer read ASamplesPerPixel
      write ASamplesPerPixel;
    property ImageType: integer read AImageType write AImageType;
  end;

var
  FormNewImage: TFormNewImage;

implementation

uses System.StrUtils, imageenio, hyieutils, PalUtils;

{$R *.dfm}

function ColorToPrettyName(const AColor: TColor): string;
{ return a string of a TColor. }
var
  cs1: string;
begin
  result := '';
  cs1 := ColorToString(AColor);
  if AnsiContainsText(cs1, 'cl') then
    Delete(cs1, 1, 2);
  result := cs1;
end;

function AddThousandSeparator(const s: string; const C: char): string;
{ Adds a specified thousand separator in the correct location in a string. }
var
  i: integer; // loops through separator position
begin
  result := s;
  i := Length(s) - 2;
  while i > 1 do
  begin
    Insert(C, result, i);
    i := i - 3;
  end;
end;

function AddDefThousandSeparator(const s: string): string;
{ Add the thousands separator from the default locale in the correct
  location in a string }
begin
  result := AddThousandSeparator(s,
    System.SysUtils.FormatSettings.ThousandSeparator);
end;

function IntegerToString(const Value: integer): string;
{ Convert value into a string with thousands seperator. }
begin
  result := AddDefThousandSeparator(IntToStr(Value));
end;

procedure TFormNewImage.RadioButton10Click(Sender: TObject);
begin
  SetFileType;
  SetPixelFormat;
  SetColor;
  SetImageDimensions;
  SetImageDescription;
end;

procedure TFormNewImage.RadioButton1Click(Sender: TObject);
begin
  SetFileType;
  SetPixelFormat;
  SetColor;
  SetImageDimensions;
  SetImageDescription;
end;

procedure TFormNewImage.RadioButton3Click(Sender: TObject);
begin
  SetFileType;
  SetPixelFormat;
  SetColor;
  SetImageDimensions;
  SetImageDescription;
end;

procedure TFormNewImage.cxRadioButton4Click(Sender: TObject);
begin
  SetFileType;
  SetPixelFormat;
  SetColor;
  SetImageDimensions;
  SetImageDescription;
end;

procedure TFormNewImage.RadioButton5Click(Sender: TObject);
begin
  SetFileType;
  SetPixelFormat;
  SetColor;
  SetImageDimensions;
  SetImageDescription;
end;

procedure TFormNewImage.RadioButton4Click(Sender: TObject);
begin
  SetFileType;
  SetPixelFormat;
  SetColor;
  SetImageDimensions;
  SetImageDescription;
end;

procedure TFormNewImage.RadioButton6Click(Sender: TObject);
begin
  SetFileType;
  SetPixelFormat;
  SetColor;
  SetImageDimensions;
  SetImageDescription;
end;

procedure TFormNewImage.RadioButton7Click(Sender: TObject);
begin
  SetFileType;
  SetPixelFormat;
  SetColor;
  SetImageDimensions;
  SetImageDescription;
end;

procedure TFormNewImage.RadioButton8Click(Sender: TObject);
begin
  SetFileType;
  SetPixelFormat;
  SetColor;
  SetImageDimensions;
  SetImageDescription;
end;

procedure TFormNewImage.ColorDepth1Click(Sender: TObject);
begin
  SetFileType;
  SetPixelFormat;
  SetColor;
  SetImageDimensions;
  SetImageDescription;
end;

procedure TFormNewImage.RadioGroup1PropertiesChange(Sender: TObject);
begin
  SetFileType;
  SetPixelFormat;
  SetColor;
  SetImageDimensions;
  SetImageDescription;
end;

procedure TFormNewImage.RadioButton2Click(Sender: TObject);
begin
  SetFileType;
  SetPixelFormat;
  SetColor;
  SetImageDimensions;
  SetImageDescription;
end;

procedure TFormNewImage.FileType1Change(Sender: TObject);
begin
  SetFileType;
  SetPixelFormat;
  SetColor;
  SetImageDimensions;
  SetImageDescription;
end;

procedure TFormNewImage.FileType1PropertiesChange(Sender: TObject);
begin
  SetFileType;
  SetPixelFormat;
  SetColor;
  SetImageDimensions;
  SetImageDescription;
end;

procedure TFormNewImage.FormCreate(Sender: TObject);
begin
  AImageWidth := 16;
  AImageHeight := 16;
  APixelFormat := pf24bit;
  ABitDepth := 24;
  AColor := clWhite;
  AFileType := IOPng;
  AFileTypeStr := 'Portable Network Graphics';
  AFileExt := '.png';
  AImageType := 0;
end;

procedure TFormNewImage.Height1PropertiesChange(Sender: TObject);
begin
  SetFileType;
  SetPixelFormat;
  SetColor;
  SetImageDimensions;
  SetImageDescription;
end;

procedure TFormNewImage.SetPixelFormat;
begin
  case ColorDepth1.ItemIndex of
    0:
      begin // 32-bit
        APixelFormat := pf32bit;
        ABitDepth := 32;
        ABitsPerSample := 8;
        ASamplesPerPixel := 4;
      end;
    1:
      begin // 24-bit
        APixelFormat := pf24bit;
        ABitDepth := 24;
        ABitsPerSample := 8;
        ASamplesPerPixel := 3;
      end;
    2:
      begin // 8-bit
        APixelFormat := pf8bit;
        ABitDepth := 8;
        ABitsPerSample := 8;
        ASamplesPerPixel := 1;
      end;
    3:
      begin // 4-bit
        APixelFormat := pf4bit;
        ABitDepth := 4;
        ABitsPerSample := 4;
        ASamplesPerPixel := 1;
      end;
    4:
      begin // 1-bit
        APixelFormat := pf1bit;
        ABitDepth := 1;
        ABitsPerSample := 1;
        ASamplesPerPixel := 1;
      end;
  end;
end;

procedure TFormNewImage.Width1PropertiesChange(Sender: TObject);
begin
  SetFileType;
  SetPixelFormat;
  SetColor;
  SetImageDimensions;
  SetImageDescription;
end;

procedure TFormNewImage.SetColor;
begin
  AColor := ColorBox1.Selected;
end;

procedure TFormNewImage.SetImageDimensions;
begin
  if RadioButton1.Checked then
  begin
    AImageWidth := 16;
    AImageHeight := 16;
    Width1.Enabled := False;
    Height1.Enabled := False;
  end
  else if RadioButton2.Checked then
  begin
    AImageWidth := 24;
    AImageHeight := 24;
    Width1.Enabled := False;
    Height1.Enabled := False;
  end
  else if RadioButton3.Checked then
  begin
    AImageWidth := 32;
    AImageHeight := 32;
    Width1.Enabled := False;
    Height1.Enabled := False;
  end
  else if RadioButton4.Checked then
  begin
    AImageWidth := 48;
    AImageHeight := 48;
    Width1.Enabled := False;
    Height1.Enabled := False;
  end
  else if RadioButton5.Checked then
  begin
    AImageWidth := 64;
    AImageHeight := 64;
    Width1.Enabled := True;
    Height1.Enabled := True;
  end
  else if RadioButton6.Checked then
  begin
    AImageWidth := 72;
    AImageHeight := 72;
    Width1.Enabled := False;
    Height1.Enabled := False;
  end
  else if RadioButton7.Checked then
  begin
    AImageWidth := 128;
    AImageHeight := 128;
    Width1.Enabled := False;
    Height1.Enabled := False;
  end
  else if RadioButton8.Checked then
  begin
    AImageWidth := StrToIntDef(Width1.Text, 32);
    AImageHeight := StrToIntDef(Height1.Text, 32);
    Width1.Enabled := True;
    Height1.Enabled := True;
  end
  else if RadioButton9.Checked then
  begin
    AImageWidth := Screen.Width;
    AImageHeight := Screen.Height;
    Width1.Enabled := True;
    Height1.Enabled := True;
  end
  else if RadioButton10.Checked then
  begin
    AImageWidth := 256;
    AImageHeight := 256;
    Width1.Enabled := False;
    Height1.Enabled := False;
  end
end;

procedure TFormNewImage.SetImageDescription;
var
  AColorString: string;
begin
  if not PalUtils.cxNameByColor(AColor, AColorString) then
    AColorString := ColorToPrettyName(AColor);
  case ColorDepth1.ItemIndex of
    0:
      begin
        Status1.Caption := '24-bit with alpha channel ' + AFileTypeStr + ' ' +
          IntegerToString(AImageWidth) + ' x ' + IntToStr(AImageHeight) +
          ' pixels ' + AColorString;
        Status1.Invalidate;
      end;
    1:
      begin
        Status1.Caption := '24-bit (16 million colors) ' + AFileTypeStr + ' ' +
          IntegerToString(AImageWidth) + ' x ' + IntToStr(AImageHeight) +
          ' pixels ' + AColorString;
        Status1.Invalidate;
      end;
    2:
      begin
        Status1.Caption := '8-bit (256 colors) ' + AFileTypeStr + ' ' +
          IntegerToString(AImageWidth) + ' x ' + IntegerToString(AImageHeight) +
          ' pixels ' + AColorString;
        Status1.Invalidate;
      end;
    3:
      begin
        Status1.Caption := '4-bit (16 colors) ' + AFileTypeStr + ' ' +
          IntegerToString(AImageWidth) + ' x ' + IntegerToString(AImageHeight) +
          ' pixels ' + AColorString;
        Status1.Invalidate;
      end;
    4:
      begin
        Status1.Caption := '1-bit (Monochrome) ' + AFileTypeStr + ' ' +
          IntegerToString(AImageWidth) + ' x ' + IntegerToString(AImageHeight) +
          ' pixels ' + AColorString;
        Status1.Invalidate;
      end;
  end;
end;

procedure TFormNewImage.SetFileType;
begin
  case FileType1.ItemIndex of
    0:
      begin
        AFileType := ioGIF;
        AFileTypeStr := 'Gif image file';
        AFileExt := '.gif';
      end;
    1:
      begin
        AFileType := ioJPEG;
        AFileTypeStr := 'Jpeg bitmap file';
        AFileExt := '.jpg';
      end;
    2:
      begin
        AFileType := ioBMP;
        AFileTypeStr := 'Windows bitmap file';
        AFileExt := '.bmp';
      end;
    3:
      begin
        AFileType := ioICO;
        AFileTypeStr := 'Windows icon file';
        AFileExt := '.ico';
      end;
    4:
      begin
        AFileType := IOPng;
        AFileTypeStr := 'Portable network graphics file';
        AFileExt := '.png';
      end;
    5:
      begin
        AFileType := ioWMF;
        AFileTypeStr := 'Windows metafile file';
        AFileExt := '.wmf';
      end;
    6:
      begin
        AFileType := ioEMF;
        AFileTypeStr := 'Enhanced windows metafile file';
        AFileExt := '.emf';
      end;
    7:
      begin
        AFileType := ioTIFF;
        AFileTypeStr := 'TIFF bitmap file';
        AFileExt := '.tif';
      end;
  end;
end;

procedure TFormNewImage.OKBtnClick(Sender: TObject);
begin
  SetFileType;
  SetPixelFormat;
  SetColor;
  SetImageDimensions;
  SetImageDescription;
end;

end.
