(* ------------------------------------------------------------------------------
  Resample                : 1.0
  Copyright  1986-2017   : Copyright Adirondack Software & Graphics
  Last Modification       : 04-15-2017
  Compiler                : Delphi Seattle
  Target Platform         : VCL Win32/Win64
  Operating System        : Windows 10
  ImageEn                 : 7.0
  This file is copyright  W W Miller, 1986-2017
  It may be used without restriction. This code distributed on an "AS IS"
  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
  ------------------------------------------------------------------------------ *)

unit uResample;
{$WARN SYMBOL_PLATFORM OFF}
{$WARN UNIT_PLATFORM OFF}

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs,
  Vcl.ComCtrls, Vcl.StdCtrls, Vcl.ExtCtrls, Vcl.Menus,
  ImageEnView, ImageEn, ImageEnIO, ImageEnProc, HYIEutils, HYIEDefs,
  IEView, iexBitmaps, iesettings, iexLayers, iexRulers;

type
  TFormResample = class(TForm)
    ImageEnView1: TImageEnView;
    ImageEnView2: TImageEnView;
    Panel1: TPanel;
    Result1: TLabel;
    Dimensions1: TLabel;
    Dimensions2: TLabel;
    Label4: TLabel;
    Filter1: TGroupBox;
    Label3: TLabel;
    Size1: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    Alignment1: TGroupBox;
    Label6: TLabel;
    Label7: TLabel;
    ComboBoxResampleFilter1: TComboBox;
    ComboBoxHorzontalAlignment1: TComboBox;
    ComboBoxVerticalAlignment1: TComboBox;
    Preview1: TButton;
    Reset1: TButton;
    Ok1: TButton;
    Cancel1: TButton;
    Undo1: TButton;
    Proportional1: TCheckBox;
    WidthEdit1: TEdit;
    HeightEdit1: TEdit;
    Width1: TUpDown;
    Height1: TUpDown;
    RadioGroup1: TRadioGroup;
    View1: TRadioGroup;
    LockPreview1: TCheckBox;
    Center1: TCheckBox;
    Apply1: TButton;
    procedure FormActivate(Sender: TObject);
    procedure Reset1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ImageEnView1MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure Preview1Click(Sender: TObject);
    procedure ImageEnView2MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure ImageEnView1MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure ImageEnView2MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure ImageEnView1ZoomIn(Sender: TObject; var NewZoom: Double);
    procedure ImageEnView2ZoomIn(Sender: TObject; var NewZoom: Double);
    procedure ImageEnView2ZoomOut(Sender: TObject; var NewZoom: Double);
    procedure Center1Click(Sender: TObject);
    procedure Undo1Click(Sender: TObject);
    procedure LockPreview1Click(Sender: TObject);
    procedure Width1PropertiesChange(Sender: TObject);
    procedure Height1PropertiesChange(Sender: TObject);
    procedure ImageEnView2ImageChange(Sender: TObject);
    procedure RadioGroup1Click(Sender: TObject);
    procedure View1Click(Sender: TObject);
    procedure Apply1Click(Sender: TObject);
    procedure ImageEnView2ViewChange(Sender: TObject; Change: Integer);
    procedure ImageEnView1ViewChange(Sender: TObject; Change: Integer);
    procedure FormResize(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure Cancel1Click(Sender: TObject);
  private
    { Private declarations }
    ADontChange: boolean;
    ImageEnView: TImageEnView;
    procedure Preview;
  public
    { Public declarations }
    Applied: boolean;
    AOrgWidth, AOrgHeight: Integer;
  published
    { Published declarations }
  end;

var
  FormResample: TFormResample;

implementation

uses Unit1;
{$R *.DFM}

function AddThousandSeparator(const AString: string; const AChar: Char): string;
{ Add a specified thousand separator in the correct location in a string. }
var
  i: Integer; { Loop through separator position }
begin
  Result := AString;
  i := Length(AString) - 2;
  while i > 1 do
  begin
    Insert(AChar, Result, i);
    i := i - 3;
  end;
end;

function AddDefThousandSeparator(const AString: string): string;
{ Add the local default thousand separator in the correct location
  in a string. }
begin
  Result := AddThousandSeparator(AString, System.SysUtils.FormatSettings.ThousandSeparator);
end;

function AddThousandSeparatorToInt(const AInteger: Integer; const AChar: Char): string;
{ Add a thousand separator to an integer and return a string }
var
  i: Integer;
  AString: string;
begin
  AString := IntToStr(AInteger);
  Result := AString;
  i := Length(AString) - 2;
  while i > 1 do
  begin
    Insert(AChar, Result, i);
    i := i - 3;
  end;
end;

function AddDefThousandSeparatorToInt(const AInteger: Integer): string;
{ Adds the thousands separator to an integer from the default locale in the correct location
  and returns a string }
begin
  Result := AddThousandSeparatorToInt(AInteger, System.SysUtils.FormatSettings.ThousandSeparator);
end;

function IntegerToString(const i: Int64): string;
{ Add the local default thousand separator in the correct
  location in a number and returns a string. }
begin
  Result := AddThousandSeparator(IntToStr(i), System.SysUtils.FormatSettings.ThousandSeparator);
end;

procedure TFormResample.FormCreate(Sender: TObject);
begin
  ImageEnView1.MouseInteract := [miZoom, miScroll];
  ImageEnView1.Cursor := 1779;
  ImageEnView1.Scrollbars := ssBoth;
  ImageEnView1.ZoomFilter := rfNone;
  ImageEnView1.Gestures.Pan.Enabled := True;
  ImageEnView1.Gestures.Zoom.Enabled := True;
  ImageEnView2.Assign(ImageEnView1);
  ImageEnView2.MouseInteract := [miZoom, miScroll];
  ImageEnView2.Cursor := 1779;
  ImageEnView2.Scrollbars := ssBoth;
  ImageEnView2.ZoomFilter := rfNone;
  ImageEnView2.Gestures.Pan.Enabled := True;
  ImageEnView2.Gestures.Zoom.Enabled := True;
  ImageEnView2.Proc.AutoUndo := False;
  ImageEnView2.Proc.UndoLimit := 99;
  Undo1.Enabled := ImageEnView2.Proc.CanUndo;
  AOrgWidth := ImageEnView1.IEBitmap.Width;
  AOrgHeight := ImageEnView1.IEBitmap.Height;
  ADontChange := False;
  Applied := False;
  ImageEnView2.Assign(ImageEnView1);
  ImageEnView1.Fit;
  ImageEnView2.Fit;
  ADontChange := False;
end;

procedure TFormResample.FormResize(Sender: TObject);
begin
  ImageEnView1.Width := (Width div 2) - 16;
  ImageEnView2.Width := (Width div 2) - 16;
  ImageEnView2.Left := ImageEnView1.Left + ImageEnView1.Width + 4;
  Result1.Left := ImageEnView2.Left;
  Dimensions1.Top := ImageEnView1.Top + ImageEnView1.Height + 8;
  Dimensions2.Left := ImageEnView2.Left;
  Dimensions2.Top := Dimensions1.Top;
  Size1.Top := Dimensions1.Top + Dimensions1.Height + 8;
  Filter1.Top := Size1.Top;
  Alignment1.Top := Size1.Top;
  Filter1.Left := ImageEnView2.Left;
  Alignment1.Left := Filter1.Left + Filter1.Width + 8;
  View1.Left := ImageEnView2.Left;
  Preview1.Left := ImageEnView2.Left;
  LockPreview1.Left := ImageEnView2.Left;
  Reset1.Left := Preview1.Left + Preview1.Width + 4;
  Undo1.Left := Reset1.Left + Reset1.Width + 4;
  LockPreview1.Left := Preview1.Left;
  View1.Top := Filter1.Top + Filter1.Height + 8;
  Preview1.Top := View1.Top + View1.Height + 8;
  Reset1.Top := Preview1.Top;
  Reset1.Left := Preview1.Left + Preview1.Width + 4;
  Undo1.Left := Reset1.Left + Reset1.Width + 4;
  Undo1.Top := Preview1.Top;
  LockPreview1.Top := Preview1.Top + Preview1.Height + 8;
  ImageEnView1.Fit;
end;

procedure TFormResample.FormShow(Sender: TObject);
begin
  ImageEnView1.Width := (Width div 2) - 16;
  ImageEnView2.Width := (Width div 2) - 16;
  ImageEnView2.Left := ImageEnView1.Left + ImageEnView1.Width + 4;
  Result1.Left := ImageEnView2.Left;
  Dimensions1.Top := ImageEnView1.Top + ImageEnView1.Height + 8;
  Dimensions2.Left := ImageEnView2.Left;
  Dimensions2.Top := Dimensions1.Top;
  Size1.Top := Dimensions1.Top + Dimensions1.Height + 8;
  Filter1.Top := Size1.Top;
  Alignment1.Top := Size1.Top;
  Filter1.Left := ImageEnView2.Left;
  Alignment1.Left := Filter1.Left + Filter1.Width + 8;
  View1.Left := ImageEnView2.Left;
  Preview1.Left := ImageEnView2.Left;
  LockPreview1.Left := ImageEnView2.Left;
  Reset1.Left := Preview1.Left + Preview1.Width + 4;
  Undo1.Left := Reset1.Left + Reset1.Width + 4;
  LockPreview1.Left := Preview1.Left;
  View1.Top := Filter1.Top + Filter1.Height + 8;
  Preview1.Top := View1.Top + View1.Height + 8;
  Reset1.Top := Preview1.Top;
  Reset1.Left := Preview1.Left + Preview1.Width + 4;
  Undo1.Left := Reset1.Left + Reset1.Width + 4;
  Undo1.Top := Preview1.Top;
  LockPreview1.Top := Preview1.Top + Preview1.Height + 8;
  ImageEnView1.Fit;
  ImageEnView2.Fit;
end;

procedure TFormResample.FormActivate(Sender: TObject);
begin
  ComboBoxResampleFilter1.ItemIndex := 0;
  Preview1.Enabled := not LockPreview1.Checked;
  AOrgWidth := ImageEnView1.IEBitmap.Width;
  AOrgHeight := ImageEnView1.IEBitmap.Height;
  Width1.Position := AOrgWidth;
  Height1.Position := AOrgHeight;
  Width1.SetFocus;
  ADontChange := False;
  Dimensions1.Caption := 'Dimensions: ' + AddDefThousandSeparatorToInt(ImageEnView1.IEBitmap.Width) + ' pixels x ' +
    AddDefThousandSeparatorToInt(ImageEnView1.IEBitmap.Height) + ' pixels';
  Dimensions2.Caption := 'Dimensions: ' + AddDefThousandSeparatorToInt(ImageEnView2.IEBitmap.Width) + ' pixels x ' +
    AddDefThousandSeparatorToInt(ImageEnView2.IEBitmap.Height) + ' pixels';
end;

procedure TFormResample.Preview;
var
  iWidth, iHeight: Integer;
begin
  ImageEnView2.Proc.SaveUndo();
  Undo1.Enabled := ImageEnView2.Proc.CanUndo;
  if Proportional1.Checked and not ADontChange then
  begin
    ADontChange := True;
    iWidth := Width1.Position;
    iHeight := Height1.Position;
    if (iWidth > 0) and (iHeight > 0) then
    begin
      ImageEnView2.Proc.Resample(iWidth, iHeight, TResampleFilter(ComboBoxResampleFilter1.ItemIndex), Proportional1.Checked);
      ImageEnView2.Update;
    end;
    ADontChange := False;
  end
  else
  begin
    ADontChange := True;
    iWidth := Width1.Position;
    iHeight := Height1.Position;
    if (iWidth > 0) and (iHeight > 0) then
    begin
      ImageEnView2.Proc.Resample(iWidth, iHeight, TResampleFilter(ComboBoxResampleFilter1.ItemIndex), Proportional1.Checked);
      ImageEnView2.Update;
    end;
    ADontChange := False;
  end;
  Dimensions2.Caption := 'Dimensions: ' + AddDefThousandSeparatorToInt(Width1.Position) + ' pixels x ' +
    AddDefThousandSeparatorToInt(Height1.Position) + ' pixels';
end;

procedure TFormResample.RadioGroup1Click(Sender: TObject);
begin
  Proportional1.Checked := False;
  AOrgWidth := ImageEnView1.IEBitmap.Width;
  AOrgHeight := ImageEnView1.IEBitmap.Height;
  Width1.Position := AOrgWidth;
  Height1.Position := AOrgHeight;

  case RadioGroup1.ItemIndex of
    0: // 320 x 568
      begin
        Width1.Position := 320;
        Height1.Position := 568;
      end;
    1: // 320 x 240
      begin
        Width1.Position := 320;
        Height1.Position := 240;
      end;
    2: // 640 x 480
      begin
        Width1.Position := 640;
        Height1.Position := 480;
      end;
    3: // 800 x 600
      begin
        Width1.Position := 800;
        Height1.Position := 600;
      end;
    4: // 960 x 640
      begin
        Width1.Position := 960;
        Height1.Position := 649;
      end;
    5: // 1024 x 768
      begin
        Width1.Position := 1024;
        Height1.Position := 768;
      end;
    6: // 1280 x 800
      begin
        Width1.Position := 1280;
        Height1.Position := 800;
      end;
    7: // 1366 x 768
      begin
        Width1.Position := 1366;
        Height1.Position := 768;
      end;
    8: // 1440 x 900
      begin
        Width1.Position := 1440;
        Height1.Position := 900;
      end;
    9: // 1600 x 900
      begin
        Width1.Position := 1600;
        Height1.Position := 900;
      end;
    10: // 1600 x 1200
      begin
        Width1.Position := 1600;
        Height1.Position := 1200;
      end;
    11: // 1680 x 1050
      begin
        Width1.Position := 1680;
        Height1.Position := 1050;
      end;
    12: // 1920 x 1080
      begin
        Width1.Position := 1920;
        Height1.Position := 1080;
      end;
    13: // 1920 x 1200
      begin
        Width1.Position := 1920;
        Height1.Position := 1200;
      end;
    14: // 2160 x 1400
      begin
        Width1.Position := 2160;
        Height1.Position := 1400;
      end;
    15: // 2560 x 1600
      begin
        Width1.Position := 2560;
        Height1.Position := 1600;
      end;
  end; // case
  Dimensions2.Caption := 'Dimensions: ' + AddDefThousandSeparatorToInt(Width1.Position) + ' pixels x ' +
    AddDefThousandSeparatorToInt(Height1.Position) + ' pixels';
  if LockPreview1.Checked then
    Preview;
end;

procedure TFormResample.View1Click(Sender: TObject);
begin
  case View1.ItemIndex of
    0:
      begin
        ImageEnView1.Zoom := 100;
        ImageEnView1.MouseInteract := [miZoom, miScroll];
        ImageEnView1.Update;
        ImageEnView1.Cursor := 1779;
        ImageEnView2.Zoom := 100;
        ImageEnView2.MouseInteract := [miZoom, miScroll];
        ImageEnView2.Update;
        ImageEnView2.Cursor := 1779;
      end;
    1:
      begin
        ImageEnView1.Fit;
        ImageEnView1.MouseInteract := [miScroll];
        ImageEnView1.Update;
        ImageEnView1.Cursor := crDefault;
        ImageEnView2.Fit;
        ImageEnView2.MouseInteract := [miScroll];
        ImageEnView2.Update;
        ImageEnView2.Cursor := crDefault;
      end;
  end;
end;

procedure TFormResample.Reset1Click(Sender: TObject);
begin
  ImageEnView2.Clear;
  ImageEnView2.Assign(ImageEnView1);
  ImageEnView2.Update;
  ADontChange := False;
  ComboBoxResampleFilter1.ItemIndex := 0;
  RadioGroup1.ItemIndex := -1;
  AOrgWidth := ImageEnView1.IEBitmap.Width;
  AOrgHeight := ImageEnView1.IEBitmap.Height;
  Width1.Position := AOrgWidth;
  Height1.Position := AOrgHeight;
  Width1.SetFocus;
  Applied := False;
end;

procedure TFormResample.ImageEnView1MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if ImageEnView1.MouseInteract = [miZoom, miScroll] then
    ImageEnView1.Cursor := 1779
  else
    ImageEnView1.Cursor := 1782;
end;

procedure TFormResample.Preview1Click(Sender: TObject);
begin
  Preview;
end;

procedure TFormResample.ImageEnView2ImageChange(Sender: TObject);
begin
  if LockPreview1.Checked then
    Dimensions2.Caption := 'Dimensions: ' + AddDefThousandSeparatorToInt(ImageEnView2.IEBitmap.Width) + ' pixels x ' +
      AddDefThousandSeparatorToInt(ImageEnView2.IEBitmap.Height) + ' pixels';
end;

procedure TFormResample.ImageEnView2MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if ImageEnView2.MouseInteract = [miZoom, miScroll] then
    ImageEnView2.Cursor := 1779
  else
    ImageEnView2.Cursor := 1782;
end;

procedure TFormResample.LockPreview1Click(Sender: TObject);
begin
  Preview1.Enabled := not LockPreview1.Checked;
  if LockPreview1.Checked then
  begin
    LockPreview1.Caption := 'Unlock';
  end
  else
  begin
    LockPreview1.Caption := 'Lock';
  end;
end;

procedure TFormResample.Width1PropertiesChange(Sender: TObject);
begin
  if ADontChange = False then
  begin
    if Proportional1.Checked and (AOrgWidth > 0) then
    begin
      ADontChange := True;
      Height1.Position := AOrgHeight * (Width1.Position) div AOrgWidth;
      ADontChange := False;
      if LockPreview1.Checked then
        Preview;
    end
    else if (not Proportional1.Checked) and (AOrgWidth > 0) then
    begin
      if LockPreview1.Checked then
        Preview;
    end;
    if LockPreview1.Checked then
      Dimensions2.Caption := 'Dimensions: ' + AddDefThousandSeparatorToInt(Width1.Position) + ' pixels x ' +
        AddDefThousandSeparatorToInt(Height1.Position) + ' pixels';
  end;
end;

procedure TFormResample.ImageEnView1MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
  ImageEnView1.Hint := 'Zoom (' + Format('%-4.2f', [ImageEnView1.Zoom]) + '%)';
  ImageEnView2.Hint := 'Zoom (' + Format('%-4.2f', [ImageEnView2.Zoom]) + '%)';
end;

procedure TFormResample.ImageEnView1ViewChange(Sender: TObject; Change: Integer);
begin
  if Change = 0 then
  begin
    ImageEnView2.ViewX := ImageEnView1.ViewX;
    ImageEnView2.ViewY := ImageEnView1.ViewY;
  end;
  if Change = 1 then
  begin
    ImageEnView2.Zoom := ImageEnView1.Zoom;
  end;
end;

procedure TFormResample.ImageEnView2MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
  ImageEnView1.Hint := 'Zoom (' + Format('%-4.2f', [ImageEnView1.Zoom]) + '%)';
  ImageEnView2.Hint := 'Zoom (' + Format('%-4.2f', [ImageEnView2.Zoom]) + '%)';
end;

procedure TFormResample.ImageEnView2ViewChange(Sender: TObject; Change: Integer);
begin
  if Change = 0 then
  begin
    ImageEnView1.ViewX := ImageEnView2.ViewX;
    ImageEnView1.ViewY := ImageEnView2.ViewY;
  end;
  if Change = 1 then
  begin
    ImageEnView1.Zoom := ImageEnView2.Zoom;
    if Round(ImageEnView2.Zoom) = Round(ImageEnView2.GetIdealZoom) then
      View1.ItemIndex := 1
    else
      View1.ItemIndex := 0;
  end;
end;

procedure TFormResample.ImageEnView1ZoomIn(Sender: TObject; var NewZoom: Double);
begin
  ImageEnView1.Cursor := 1779;
end;

procedure TFormResample.ImageEnView2ZoomIn(Sender: TObject; var NewZoom: Double);
begin
  ImageEnView2.Cursor := 1779;
end;

procedure TFormResample.ImageEnView2ZoomOut(Sender: TObject; var NewZoom: Double);
begin
  ImageEnView2.Cursor := 1778;
end;

procedure TFormResample.Apply1Click(Sender: TObject);
begin
  if Assigned(Form1.PageControl1.ActivePage) then
  begin
    ImageEnView := TImageEnView(Form1.PageControl1.ActivePage.Controls[0]);
    if Assigned(ImageEnView) then
    begin
      Screen.Cursor := crHourGlass;
      try
        ImageEnView.Proc.SaveUndo();
        Form1.Undo1.Enabled := ImageEnView.Proc.CanUndo;
        ImageEnView1.IEBitmap.Assign(ImageEnView2.IEBitmap);
        ImageEnView1.IO.Params.Assign(ImageEnView2.IO.Params);
        ImageEnView1.Update;
        ImageEnView1.Bitmap.Modified := True;
        Applied := True;
        Form1.StatusBar1.Panels[2].Text := 'Dimensions: ' + AddDefThousandSeparatorToInt(Form1.ImageEnView.IEBitmap.Width) +
          ' pixels x ' + AddDefThousandSeparatorToInt(Form1.ImageEnView.IEBitmap.Height) + ' pixels';
      finally
        Screen.Cursor := crDefault;
      end;
    end;
  end;
end;

procedure TFormResample.Cancel1Click(Sender: TObject);
begin
  if Assigned(Form1.PageControl1.ActivePage) then
  begin
    ImageEnView := TImageEnView(Form1.PageControl1.ActivePage.Controls[0]);
    if Assigned(ImageEnView) then
    begin
      ImageEnView.Proc.Undo();
      Form1.Undo1.Enabled := Form1.ImageEnView.Proc.CanUndo;
      ImageEnView.Bitmap.Modified := ImageEnView.Proc.CanUndo;
      Form1.StatusBar1.Panels[2].Text := 'Dimensions: ' + AddDefThousandSeparatorToInt(ImageEnView.IEBitmap.Width) +
        ' pixels x ' + AddDefThousandSeparatorToInt(ImageEnView.IEBitmap.Height) + ' pixels';
    end;
  end;
end;

procedure TFormResample.Center1Click(Sender: TObject);
begin
  ImageEnView1.Center := Center1.Checked;
  ImageEnView2.Center := Center1.Checked;
end;

procedure TFormResample.Height1PropertiesChange(Sender: TObject);
begin
  if ADontChange = False then
  begin
    if Proportional1.Checked and (AOrgHeight > 0) then
    begin
      ADontChange := True;
      Width1.Position := (AOrgWidth * Height1.Position) div AOrgHeight;
      ADontChange := False;
      if LockPreview1.Checked then
        Preview;
    end
    else if (not Proportional1.Checked) and (AOrgHeight > 0) then
    begin
      if LockPreview1.Checked then
        Preview;
    end;
    if LockPreview1.Checked then
      Dimensions2.Caption := 'Dimensions: ' + AddDefThousandSeparatorToInt(Width1.Position) + ' pixels x ' +
        AddDefThousandSeparatorToInt(Height1.Position) + ' pixels';
  end;
end;

procedure TFormResample.Undo1Click(Sender: TObject);
begin
  if Assigned(Form1.PageControl1.ActivePage) then
  begin
    ImageEnView := TImageEnView(Form1.PageControl1.ActivePage.Controls[0]);
    if Assigned(ImageEnView) then
    begin
      ImageEnView2.Proc.Undo;
      ImageEnView2.Proc.ClearUndo;
      ImageEnView2.Update;
      Undo1.Enabled := ImageEnView2.Proc.CanUndo;
      Dimensions2.Caption := 'Dimensions: ' + AddDefThousandSeparatorToInt(ImageEnView2.IEBitmap.Width) + ' pixels x ' +
        IntToStr(ImageEnView2.IEBitmap.Height) + ' pixels';
      ImageEnView.Proc.Undo;
      ImageEnView.Proc.ClearUndo;
      ImageEnView.Update;
      Form1.Undo1.Enabled := ImageEnView.Proc.CanUndo;
      ImageEnView.Bitmap.Modified := ImageEnView.Proc.CanUndo;
      Form1.StatusBar1.Panels[2].Text := 'Dimensions: ' + AddDefThousandSeparatorToInt(ImageEnView.IEBitmap.Width) +
        ' pixels x ' + AddDefThousandSeparatorToInt(ImageEnView.IEBitmap.Height) + ' pixels';
    end;
  end;
end;

end.
